import docopt


class ValidationException(Exception):
    pass


def transform(args, transforms):
    for key, function in transforms.items():
        try:
            arg = args[key]
        except KeyError:
            raise ValidationException(
                '%s is not a valid argument for transformation.' % key
            )

        if arg:
            try:
                arg = function(arg)
                args[key] = arg
            except Exception as e:
                raise ValidationException(
                    'Could not transform parameter %s. Error was %s' % (
                        key,
                        str(e)
                    )
                )


def validate(args, validations):
    for key, function in validations.items():
        try:
            arg = args[key]
        except KeyError:
            raise ValidationException(
                '%s is not a valid argument for validation.' % key
            )

        if arg:
            try:
                success = function(arg)
                if not success:
                    raise Exception
            except Exception as e:
                raise ValidationException(
                    'Could not validate parameter %s. Error was %s' % (
                        key,
                        str(e)
                    )
                )


# pylint: disable=W0622
def docopt_wrapper(doc, argv=None, help=True, version=None,
                   options_first=False, transforms=None, validations=None):
    args = docopt.docopt(doc, argv, help, version, options_first)

    if validations:
        validate(args, validations)

    if transforms:
        transform(args, transforms)

    return args
